<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2010 Marc Hoersken <info@marc-hoersken.de>
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

unset($MCONF);

require_once('conf.php');
require_once($BACK_PATH.'init.php');
if (version_compare(TYPO3_branch, '6.2', '<')) {
	require_once($BACK_PATH.'template.php');
	require_once(PATH_t3lib.'class.t3lib_scbase.php');
}

$LANG->includeLLFile('EXT:mh_code_highlighter/mod1/locallang.xml');
$LANG->includeLLFile('EXT:mh_code_highlighter/locallang_db.xml');
$BE_USER->modAccess($MCONF,1);
$TBE_TEMPLATE->backPath = $BACK_PATH;

/**
 * Module 'GeSHi Syntax' for the 'mh_code_highlighter' extension.
 *
 * @author Marc Hoersken <info@marc-hoersken.de>
 * @package TYPO3
 * @subpackage tx_mhcodehighlighter
 */
class tx_mhcodehighlighter_module1 extends t3lib_SCbase {
	var $pageinfo;
	 
	/**
	 * Initializes the Module
	 *
	 * @return void
	 */
	function init() {
		global $BE_USER,$LANG,$BACK_PATH,$TCA_DESCR,$TCA,$CLIENT,$TYPO3_CONF_VARS,$TYPO3_DB;
		 
		parent::init();
		 
		$this->MCONF = $GLOBALS['MCONF'];
		$this->extKey = 'mh_code_highlighter';
		$this->extResSyntax = '../res/';
		$this->extMirrorSyntax = 'https://stuff.marc-hoersken.de/';
		 
		if (t3lib_div::_GP('clear_all_cache')) {
			$this->include_once[] = PATH_t3lib.'class.t3lib_tcemain.php';
		}
		if (t3lib_div::_GP('tx_mhcodehighlighter')) {
			$vars = t3lib_div::_GP('tx_mhcodehighlighter');
			if ($vars['syntax']) {
				$this->syntax = $vars['syntax'];
				$this->syntax_quoted = $TYPO3_DB->fullQuoteStr($this->syntax, 'tx_mhcodehighlighter_languages');
			}
			if ($vars['language']) {
				$this->language = $vars['language'];
			}
			if ($vars['cmd']) {
				$this->cmd = $vars['cmd'];
			}
			if ($vars['upload']) {
				$this->upload = $vars['upload'];
			}
			if ($vars['uploadOverwrite']) {
				$this->uploadOverwrite = $vars['uploadOverwrite'];
			}
			if ($vars['online_check']) {
				$this->online_check = $vars['online_check'];
				$url = $this->extMirrorSyntax.'geshi.mirror.json.php';
				$mirrorArray = json_decode(t3lib_div::getURL($url));
				if (is_array($mirrorArray)) {
					$this->extMirrorSyntax = $mirrorArray[rand(0, (count($mirrorArray)-1))];
				}
			}
			if ($vars['online_update']) {
				$this->online_update = $vars['online_update'];
				$url = $this->extMirrorSyntax.'geshi.mirror.json.php';
				$mirrorArray = json_decode(t3lib_div::getURL($url));
				if (is_array($mirrorArray)) {
					$this->extMirrorSyntax = $mirrorArray[rand(0, (count($mirrorArray)-1))];
				}
			}
		}
		if ($this->cmd == 'download') {
			if (file_exists($this->extResSyntax.$this->syntax)) {
				$file = file_get_contents($this->extResSyntax.$this->syntax);
				$filedata = array(0 => base64_encode($file), 1 => $this->syntax, 2 => md5($file));
				$filename = str_replace('.php', '.t3s', $this->syntax);
				header('Content-Type: application/octet-stream');
				header('Content-Disposition: attachment; filename='.$filename);
				die(json_encode($filedata));
			}
		}
		if (is_array($_FILES['upload_t3s_file']) && !empty($_FILES['upload_t3s_file']['name'])) {
			$this->upload_t3s_file = array();
			$this->upload_t3s_file['name'] = t3lib_div::upload_to_tempfile($_FILES['upload_t3s_file']['tmp_name']);
			$this->upload_t3s_file['size'] = $_FILES['upload_t3s_file']['size'];
		}
	}
	 
	/**
	 * Adds items to the->MOD_MENU array. Used for the function menu selector.
	 *
	 * @return void
	 */
	function menuConfig() {
		global $LANG;
		$this->MOD_MENU = Array (
			'function' => Array (
				'1' => $LANG->getLL('function1'),
				'2' => $LANG->getLL('function2'),
				'3' => $LANG->getLL('function3'),
			)
		);
		parent::menuConfig();
	}
	 
	/**
	 * Main function of the module. Write the content to $this->content
	 * If you chose "web" as main module, you will need to consider the $this->id parameter which will contain the uid-number of the page clicked in the page tree
	 *
	 * @return	[type]		...
	 */
	function main()	{
		global $BE_USER,$LANG,$BACK_PATH,$TCA_DESCR,$TCA,$CLIENT,$TYPO3_CONF_VARS;

		// Access check!
		// The page will show only if there is a valid page and if this page may be viewed by the user
		$this->pageinfo = t3lib_BEfunc::readPageAccess($this->id,$this->perms_clause);
		$access = is_array($this->pageinfo) ? 1 : 0;

		if (($this->id && $access) || ($BE_USER->user['admin'] && !$this->id))	{

				// Draw the header.
			$this->doc = t3lib_div::makeInstance('bigDoc');
			$this->doc->backPath = $BACK_PATH;
			$this->doc->form='<form action="" method="POST" enctype="multipart/form-data">';

				// JavaScript
			$this->doc->JScode = '
				<script language="javascript" type="text/javascript">
					script_ended = 0;
					function jumpToUrl(URL)	{
						document.location = URL;
					}
				</script>
			';
			$this->doc->postCode='
				<script language="javascript" type="text/javascript">
					script_ended = 1;
					if (top.fsMod) top.fsMod.recentIds["web"] = 0;
				</script>
			';
			 
			$headerSection = $this->doc->getHeader('pages',$this->pageinfo,$this->pageinfo['_thePath']).'<br />'.$LANG->sL('LLL:EXT:lang/locallang_core.xml:labels.path').': '.t3lib_div::fixed_lgd_cs($this->pageinfo['_thePath'],50);

			$this->content.=$this->doc->startPage($LANG->getLL('title'));
			$this->content.=$this->doc->header($LANG->getLL('title'));
			$this->content.=$this->doc->spacer(5);
			$this->content.=$this->doc->section('',$this->doc->funcMenu($headerSection,t3lib_BEfunc::getFuncMenu($this->id,'SET[function]',$this->MOD_SETTINGS['function'],$this->MOD_MENU['function'])));
			$this->content.=$this->doc->divider(5);
			
			// Print first content:
			$this->printTmpContent();
			
			// Render content:
			$this->moduleContent();
			
			// ShortCut
			if ($BE_USER->mayMakeShortcut())	{
				$this->content.=$this->doc->spacer(20).$this->doc->section('',$this->doc->makeShortcutIcon('id',implode(',',array_keys($this->MOD_MENU)),$this->MCONF['name']));
			}

			$this->content.=$this->doc->spacer(10);
			$this->printTmpContent();
		} else {
				// If no access or if ID == zero

			$this->doc = t3lib_div::makeInstance('bigDoc');
			$this->doc->backPath = $BACK_PATH;

			$this->content.=$this->doc->startPage($LANG->getLL('title'));
			$this->content.=$this->doc->header($LANG->getLL('title'));
			$this->content.=$this->doc->spacer(5);
			$this->content.=$this->doc->spacer(10);
			 
			$this->printTmpContent();
		}
	}
	 
	/**
	 * Prints out the module HTML
	 *
	 * @return void
	 */
	function flushContent() {
		echo $this->content;
		$this->content = '';
		flush();
	}
	 
	/**
	 * Prints out the module HTML
	 *
	 * @return void
	 */
	function printTmpContent() {
		echo $this->content;
		$this->content = '';
	}
	 
	/**
	 * Prints out the module HTML
	 *
	 * @return void
	 */
	function printContent() {
		$this->content .= $this->doc->endPage();
		echo $this->content;
	}
	 
	/**
	 * Prints out the progress
	 *
	 * @param integer  $value: Percentage done
	 * @param integer  $valuePrev: Previous percentage
	 * @param string  $type: check/update
	 * @param string  $geshiLang: Syntaxname
	 * @param string  $geshiStatus: Status of the syntax
	 * @param string  $bgColor: Backgroundcolor of the cell
	 * @param boolean  $showRow: Print row to table
	 * @return integer
	 */
	function onlineProgress($value, $valuePrev, $type, $geshiLang, $geshiStatus, $bgColor = '#69a550', $showRow = true) {
		global $LANG;

		$i = intval($value);
		$j = intval(100 - $i);

		$this->content .= '<script language="javascript" type="text/javascript">';

		if ($valuePrev != $i) {
			$this->content .= 'document.getElementById("progress-bar").style.width = "'.$i.'%";	document.getElementById("transparent-bar").style.width = "'.$j.'%"; ';
		}

		$this->content .= 'document.getElementById("progress-message").firstChild.data="'.$LANG->getLL('online_'.$type.'ing').' \"'.$geshiLang.'\" ...";</script>';

		if ($showRow) {
			$this->content .= '<tr class="bgColor4"><td>'.$geshiLang.'</td><td style="background-color:'.$bgColor.'">'.$geshiStatus.'</td></tr>';
		}

		$this->flushContent();

		return $i;
	}
	 
	/**
	 * Checks for updated syntax files
	 *
	 * @return boolean
	 */
	function onlineCheck() {
		global $LANG,$TYPO3_DB;

		$url = $this->extMirrorSyntax.'geshi.list.json.php';
		$syntaxArray = json_decode(t3lib_div::getURL($url));

		$i = 0;
		$j = 0;
		$k = count($syntaxArray);

		if (is_array($syntaxArray)) {
			foreach ($syntaxArray as $m => $syntax) {
				if (is_array($syntax)) {
					$syntaxName = $syntax[0];
					$syntaxHash = $syntax[1];
					unset($language_data);
					if (@include($this->extResSyntax.$syntaxName)) {
						if (is_array($language_data)) {
							$language_hash = substr(md5(json_encode($language_data)), 0, 10);
						}
					}
					if ($syntaxHash != $language_hash) {
						$syntaxStatus = 'UPD';
						$bgColor = '#FFFF00';
					} else {
						$syntaxStatus = 'OK';
						$bgColor = '#69A550';
					}
					$i++;
					if ($k > 0) {
						$percent = (($i / $k) * 100);
					} else {
						$percent = 100;
					}
					$percentTemp = $this->onlineProgress($percent, $percentTemp, 'check', $syntaxName, $syntaxStatus, $bgColor, true);
				} else {
					$k--;
					if ($k > 0) {
						$percent = (($i / $k) * 100);
					} else {
						$percent = 100;
					}
					$percentTemp = $this->onlineProgress($percent, $percentTemp, 'check', 0, 0, 0, false);
				}
			}
			return true;
		} else {
			$this->content .= '<script language="javascript" type="text/javascript">document.getElementById("progress-message").firstChild.data="'.$LANG->getLL('online_timeout').'.";</script>';
			$this->flushContent();
			return false;
		}
	}
	 
	/**
	 * Downloads and updates syntax files
	 *
	 * @return boolean
	 */
	function onlineUpdate() {
		global $LANG,$TYPO3_DB;

		$url = $this->extMirrorSyntax.'geshi.list.json.php';
		$syntaxArray = json_decode(t3lib_div::getURL($url));

		$i = 0;
		$j = 0;
		$k = count($syntaxArray);

		if (is_array($syntaxArray)) {
			foreach ($syntaxArray as $m => $syntax) {
				if (is_array($syntax)) {
					$syntaxName = $syntax[0];
					$syntaxHash = $syntax[1];
					unset($language_data);
					if (@include($this->extResSyntax.$syntaxName)) {
						if (is_array($language_data)) {
							$language_hash = substr(md5(json_encode($language_data)), 0, 10);
						}
					}
					if ($syntaxHash != $language_hash) {
						$syntaxStatus = 'FAILED';
						$bgColor = '#FF0000';
						$url = $this->extMirrorSyntax.'geshi.file.json.php?syntax='.$syntaxName;
						$syntaxFileArray = json_decode(t3lib_div::getURL($url));
						$syntaxFile = base64_decode($syntaxFileArray[0]);
						$syntaxFileHash = $syntaxFileArray[1];
						if (md5($syntaxFile) == $syntaxFileHash) {
							if ($fh = fopen($this->extResSyntax.$syntaxName, 'w+')) {
								if (fwrite($fh, $syntaxFile)) {
									$syntaxStatus = 'UPD';
									$bgColor = '#69A550';
								}
								fclose($fh);
							}
						}
					} else {
						$syntaxStatus = 'OK';
						$bgColor = '#69A550';
					}
					$i++;
					if ($k > 0) {
						$percent = (($i / $k) * 100);
					} else {
						$percent = 100;
					}
					$percentTemp = $this->onlineProgress($percent, $percentTemp, 'update', $syntaxName, $syntaxStatus, $bgColor, true);
				} else {
					$k--;
					if ($k > 0) {
						$percent = (($i / $k) * 100);
					} else {
						$percent = 100;
					}
					$percentTemp = $this->onlineProgress($percent, $percentTemp, 'update', 0, 0, 0, false);
				}
			}
			return true;
		} else {
			$this->content .= '<script language="javascript" type="text/javascript">document.getElementById("progress-message").firstChild.data="'.$LANG->getLL('online_timeout').'.";</script>';
			$this->flushContent();
			return false;
		}
	}
	 
	/**
	 * Strips selected tags from string
	 *
	 * @param string  $text: String to strip tags from
	 * @param array  $tags: Array of tags to strip
	 * @return string
	 */
	function stripSelectedTags($text, $tags = array()) {
		$args = func_get_args();
		$text = array_shift($args);
		$tags = func_num_args() > 2 ? array_diff($args, array($text)) : (array)$tags;
		foreach ($tags as $tag) {
			while (preg_match('/<'.$tag.'(|\W[^>]*)>(.*)<\/'. $tag .'>/iusU', $text, $found)) {
				$text = str_replace($found[0], $found[2], $text);
			}
		}
		return preg_replace('/(<('.join('|', $tags).')(|\W.*)\/>)/iusU', '', $text);
	}
	 
	/**
	 * Generates the module content
	 *
	 * @return void
	 */
	function moduleContent() {
		global $LANG,$TYPO3_DB,$BE_USER;
		 
		if ($this->cmd == 'remove') {
			$res = $TYPO3_DB->exec_SELECTquery('*', 'tx_mhcodehighlighter_languages', 'geshi_file = '.$this->syntax_quoted);
			$num = $TYPO3_DB->sql_num_rows($res);
			if ($num > 0) {
				$TYPO3_DB->exec_DELETEquery('tx_mhcodehighlighter_languages', 'geshi_file = '.$this->syntax_quoted);
				$BE_USER->simplelog('Removed syntax \'<em>'.$this->syntax.'</em>\' from database', $this->extKey, 0);
			}
		}
		else if ($this->cmd == 'load') {
			$res = $TYPO3_DB->exec_SELECTquery('*', 'tx_mhcodehighlighter_languages', 'geshi_file = '.$this->syntax_quoted);
			$num = $TYPO3_DB->sql_num_rows($res);
			if (($num == 0) && (file_exists($this->extResSyntax.$this->syntax))) {
				$syntaxArray = array();
				$syntaxArray['geshi_lang'] = $this->language;
				$syntaxArray['geshi_file'] = $this->syntax;
				$syntaxArray['tstamp'] = filemtime($this->extResSyntax.$this->syntax);
				$syntaxArray['crdate'] = filemtime($this->extResSyntax.$this->syntax);
				$TYPO3_DB->exec_INSERTquery('tx_mhcodehighlighter_languages', $syntaxArray);
				$BE_USER->simplelog('Installed syntax \'<em>'.$this->syntax.'</em>\' into database', $this->extKey, 0);
			}
		}
		else if ($this->cmd == 'info') {
			$res = $TYPO3_DB->exec_SELECTquery('*', 'tx_mhcodehighlighter_languages', 'geshi_file = '.$this->syntax_quoted);
			$num = $TYPO3_DB->sql_num_rows($res);
			if (($num > 0) && file_exists($this->extResSyntax.$this->syntax)) {
				unset($language_data);
				require($this->extResSyntax.$this->syntax);
				$arrayBrowser = t3lib_div::makeInstance('t3lib_arrayBrowser');
				$arrayBrowser->varName = '$LANGUAGE_DATA';
				$arrayBrowser->dontLinkVar = true;
				$node = t3lib_div::_GET('node');
				if (is_array($node)) {
					$this->MOD_SETTINGS['node_'.$this->syntax] = $arrayBrowser->depthKeys($node, $this->MOD_SETTINGS['node_'.$this->syntax]);
					$GLOBALS['BE_USER']->pushModuleData($this->MCONF['name'], $this->MOD_SETTINGS);
				}
				$arrayBrowser->depthKeys = $this->MOD_SETTINGS['node_'.$this->syntax];
				$tree = $arrayBrowser->tree($language_data, '', '');
				$tree = str_replace('index.php?', 'index.php?tx_mhcodehighlighter[syntax]='.$this->syntax.'&amp;tx_mhcodehighlighter[cmd]=info&amp;', $tree);
				$content .= '<table border="0" cellpadding="2" cellspacing="1" width="100%"><tr class="bgColor2"><td align="right"><a href="index.php" style="display: block; font-weight: bold;">'.$LANG->getLL('close').'&nbsp;</a></td></tr><tr class="bgColor5"><td>';
				$content .= $tree;
				$content .= '</td></tr><tr class="bgColor2"><td align="right"><a href="index.php" style="display: block; font-weight: bold;">'.$LANG->getLL('close').'&nbsp;</a></td></tr></table>';
			}
		}
		 
		unset($language_data);
		 
		switch((string)$this->MOD_SETTINGS['function']) {
			case 1:
				$content .= '
					<table border="0" cellpadding="2" cellspacing="1" width="100%">
						<tr class="bgColor5" style="font-weight: bold;">
							<td width="1%"><img src="clear.gif" width="18" height="1" alt="" /></td>
							<td>'.$LANG->getLL('syntax_title').':</td>
							<td>'.$LANG->getLL('syntax_date').':</td>
							<td width="1%">'.$LANG->getLL('syntax_update').':</td>
							<td width="1%">'.$LANG->getLL('syntax_keywords').':</td>
						</tr>
				';
			 
				$res = $TYPO3_DB->exec_SELECTquery('*', 'tx_mhcodehighlighter_languages', '', '', 'geshi_lang');
				$num = $TYPO3_DB->sql_num_rows($res);
				if ($num > 0) {
					while ($row = $TYPO3_DB->sql_fetch_assoc($res)) {
						if (file_exists($this->extResSyntax.$row['geshi_file'])) {
							unset($language_data);
							require($this->extResSyntax.$row['geshi_file']);
							$language_datetime = date('F d Y H:i:s', filemtime($this->extResSyntax.$row['geshi_file']));
							$language_keywords = count($language_data['KEYWORDS']);
							$content .= '
								<tr>
									<td class="bgColor" align="center"><a href="index.php?tx_mhcodehighlighter[syntax]='.$row['geshi_file'].'&amp;tx_mhcodehighlighter[cmd]=remove"><img src="res/uninstall.gif" width="16" height="16" title="'.$LANG->getLL('uninstall').'" align="top" alt="" /></a></td>
									<td nowrap="nowrap" class="bgColor"><a href="index.php?tx_mhcodehighlighter[syntax]='.$row['geshi_file'].'&amp;tx_mhcodehighlighter[cmd]=info">'.$row['geshi_lang'].'</a></td>
									<td nowrap="nowrap" class="bgColor">'.$language_datetime.'</td>
									<td class="bgColor" align="center"><a href="index.php?tx_mhcodehighlighter[syntax]='.$row['geshi_file'].'&amp;tx_mhcodehighlighter[cmd]=download"><img src="res/download.png" title="'.$LANG->getLL('download').'" alt="" /></a></td>
									<td class="bgColor" align="center">'.$language_keywords.'</td>
								</tr>
							';
						}
					}
				}
			 
				$content .= '
					</table>
				';
			 
				$this->content .= $this->doc->section($LANG->getLL('function1').':', $content, 0, 1);
			break;

			case 2:
				$content .= '
					<table border="0" cellpadding="2" cellspacing="1" width="100%">
						<tr class="bgColor5" style="font-weight: bold;">
							<td width="1%"><img src="clear.gif" width="18" height="1" alt="" /></td>
							<td>'.$LANG->getLL('syntax_title').':</td>
							<td>'.$LANG->getLL('syntax_date').':</td>
							<td width="1%">'.$LANG->getLL('syntax_update').':</td>
							<td width="1%">'.$LANG->getLL('syntax_keywords').':</td>
						</tr>
				';
			 
				if (is_dir($this->extResSyntax)) {
					if ($dh = opendir($this->extResSyntax)) {
						while (($file = readdir($dh)) !== false) {
							if ($file != '..' && $file != '.') {
								$file_quoted = $TYPO3_DB->fullQuoteStr($file, 'tx_mhcodehighlighter_languages');
								$res = $TYPO3_DB->exec_SELECTquery('*', 'tx_mhcodehighlighter_languages', 'geshi_file = '.$file_quoted);
								$num = $TYPO3_DB->sql_num_rows($res);
								if ($num > 0) {
									$row = $TYPO3_DB->sql_fetch_assoc($res);
									unset($language_data);
									require($this->extResSyntax.$row['geshi_file']);
									$language_datetime = date('F d Y H:i:s', filemtime($this->extResSyntax.$row['geshi_file']));
									$language_keywords = count($language_data['KEYWORDS']);
									$content .= '
										<tr>
											<td class="bgColor" align="center"><a href="index.php?tx_mhcodehighlighter[syntax]='.$file.'&amp;tx_mhcodehighlighter[cmd]=remove"><img src="res/uninstall.gif" width="16" height="16" title="'.$LANG->getLL('uninstall').'" align="top" alt="" /></a></td>
											<td nowrap="nowrap" class="bgColor"><a href="index.php?tx_mhcodehighlighter[syntax]='.$file.'&amp;tx_mhcodehighlighter[cmd]=info">'.$row['geshi_lang'].'</a></td>
											<td nowrap="nowrap" class="bgColor">'.$language_datetime.'</td>
											<td class="bgColor" align="center"><a href="index.php?tx_mhcodehighlighter[syntax]='.$file.'&amp;tx_mhcodehighlighter[cmd]=download"><img src="res/download.png" title="'.$LANG->getLL('download').'" alt="" /></a></td>
											<td class="bgColor" align="center">'.$language_keywords.'</td>
										</tr>
									';
								} else {
									unset($language_data);
									require($this->extResSyntax.$file);
									$language_datetime = date('F d Y H:i:s', filemtime($this->extResSyntax.$file));
									$language_keywords = count($language_data['KEYWORDS']);
									$content .= '
										<tr>
											<td class="bgColor" align="center"><a href="index.php?tx_mhcodehighlighter[syntax]='.$file.'&amp;tx_mhcodehighlighter[language]='.$language_data['LANG_NAME'].'&amp;tx_mhcodehighlighter[cmd]=load"><img src="res/install.gif" width="16" height="16" title="'.$LANG->getLL('install').'" align="top" alt="" /></a></td>
											<td nowrap="nowrap" class="bgColor"><a href="index.php?tx_mhcodehighlighter[syntax]='.$file.'&amp;tx_mhcodehighlighter[cmd]=info">'.$language_data['LANG_NAME'].'</a></td>
											<td nowrap="nowrap" class="bgColor">'.$language_datetime.'</td>
											<td class="bgColor" align="center"><a href="index.php?tx_mhcodehighlighter[syntax]='.$file.'&amp;tx_mhcodehighlighter[cmd]=download"><img src="res/download.png" title="'.$LANG->getLL('download').'" alt="" /></a>&nbsp;<img src="clear.gif" width="16" height="1" alt="" /></td>
											<td class="bgColor" align="center">'.$language_keywords.'</td>
										</tr>
									';
								}
							}
						}
						closedir($dh);
					}
				}
			 
				$content .= '
					</table>
				';
			 
				$this->content .= $this->doc->section($LANG->getLL('function2').':', $content, 0, 1);
			break;

			case 3:
				$this->content .= $this->doc->section($LANG->getLL('function3.1').':', $content, 0, 1);

				if (!empty($this->upload)) {
					$file = t3lib_div::getURL($this->upload_t3s_file['name']);
					$temp = t3lib_div::unlink_tempfile($this->upload_t3s_file['name']);
					$syntaxFileArray = json_decode($file);
					if (is_null($syntaxFileArray)) {
						if (version_compare(PHP_VERSION, '7.0.0') >= 0) {
							$syntaxFileArray = unserialize($file, array('allowed_classes' => false));
						} else {
							$syntaxFileArray = unserialize($file);
						}
					}
					if (is_array($syntaxFileArray) && count($syntaxFileArray) == 3) {
						$syntaxFile = base64_decode($syntaxFileArray[0]);
						$syntaxFileName = $syntaxFileArray[1];
						$syntaxFileHash = $syntaxFileArray[2];
					}
					if (is_array($syntaxFileArray) && md5($syntaxFile) == $syntaxFileHash) {
						if (!file_exists($this->extResSyntax.$syntaxFileName) || !empty($this->uploadOverwrite)) {
							if ($fh = fopen($this->extResSyntax.$syntaxFileName, 'w+')) {
								if (fwrite($fh, $syntaxFile)) {
									$this->content .= '
										<a href="index.php" class="typo3-goBack"><img src="res/goback.gif" width="16" height="16" alt="" /> '.$LANG->getLL('import_goback').'</a><br />'.$LANG->getLL('import_success').': '.$this->extResSyntax.$syntaxFileName.'<br />
									';

									$syntaxFileName_quoted = $TYPO3_DB->fullQuoteStr($syntaxFileName, 'tx_mhcodehighlighter_languages');
									$res = $TYPO3_DB->exec_SELECTquery('uid', 'tx_mhcodehighlighter_languages', 'geshi_file = '.$syntaxFileName_quoted);
									$num = $TYPO3_DB->sql_num_rows($res);
									if ($num == 0) {
										$this->content .= '
											<h3>'.$LANG->getLL('import_install').':</h3>
											<a href="index.php?SET[function]=2&amp;tx_mhcodehighlighter[syntax]='.$syntaxFileName.'&amp;tx_mhcodehighlighter[cmd]=load"><img src="res/install.gif" width="16" height="16" title="'.$LANG->getLL('import_install').'" align="top" alt="" /> '.$LANG->getLL('import_install').'</a>
										';
									} else {
										$this->content .= '
											<h3>'.$LANG->getLL('import_success').':</h3>
										';
									}
								}
								fclose($fh);
							} else {
								$this->content .= '
									<span class="typo3-red">'.str_replace(array('###path###', '###PATH###'), $this->extResSyntax.$syntaxFileName, $LANG->getLL('import_notwriteable')).'</span><br /><br /><a href="index.php" class="typo3-goBack"><img src="res/goback.gif" width="16" height="16" alt="" /> '.$LANG->getLL('import_goback').'</a>
								';
							}
						} else {
							$this->content .= '
								<span class="typo3-red">'.str_replace(array('###path###', '###PATH###'), $this->extResSyntax.$syntaxFileName, $LANG->getLL('import_exists')).'</span><br /><br /><a href="index.php" class="typo3-goBack"><img src="res/goback.gif" width="16" height="16" alt="" /> '.$LANG->getLL('import_goback').'</a>
							';
						}
					} else {
						$this->content .= '
							<span class="typo3-red">'.$LANG->getLL('import_invalid').'</span><br /><br /><a href="index.php" class="typo3-goBack"><img src="res/goback.gif" width="16" height="16" alt="" /> '.$LANG->getLL('import_goback').'</a>
						';
					}
				} else {
					$this->content .= '
						Upload packaged syntax file (.t3s):<br />
						<input type="file" size="60" name="upload_t3s_file" /><br />
						<input type="checkbox" value="1" name="tx_mhcodehighlighter[uploadOverwrite]" id="checkUploadOverwrite" /> <label for="checkUploadOverwrite">Overwrite any existing syntax!</label><br />
						<input type="submit" name="tx_mhcodehighlighter[upload]" value="Upload packaged syntax file" /><br />
					';
				 
					$this->content .= $this->doc->section($LANG->getLL('function3.2').':', $content, 0, 1);
					$this->content .= '
						<input type="submit" name="tx_mhcodehighlighter[online_check]" value="'.$LANG->getLL('online_check').'" />&nbsp;<input type="submit" name="tx_mhcodehighlighter[online_update]" value="'.$LANG->getLL('online_update').'" />
						<br />
					';
				 
					if (!empty($this->online_check)) {
						$this->content .= '
							<br />
							<p id="progress-message">'.$LANG->getLL('online_wait').' ...</p>
							<script language="javascript" type="text/javascript">
								function timeoutRep() {
									if (document.getElementById("progress-message").firstChild.data == "'.$LANG->getLL('online_wait').' ...") {
										document.getElementById("progress-message").firstChild.data="'.$LANG->getLL('online_timeout').'.";
									}
								}
								setTimeout("timeoutRep()", 10000);
							</script>
							<br />
							<div style="width:100%; height:20px; border: 1px solid black;">
								<div id="progress-bar" style="float: left; width: 0%; height: 20px; background-color:green;">&nbsp;</div>
								<div id="transparent-bar" style="float: left; width: 100%; height: 20px; background-color:#FEFEFE;">&nbsp;</div>
							</div>
							<script language="javascript" type="text/javascript">
								document.getElementById("progress-bar").firstChild.data="";
								document.getElementById("transparent-bar").firstChild.data="";
							</script>
							<br />
							<br />
							<table border="0" cellpadding="2" cellspacing="2">
								<tr class="bgColor2">
									<td>'.$LANG->getLL('syntax_title').'</td>
									<td>'.$LANG->getLL('syntax_status').'</td>
								</tr>
						';

						$this->flushContent();
					 
						if ($this->onlineCheck()) {
							$this->content .= '
								<script language="javascript" type="text/javascript">
									document.getElementById("progress-message").firstChild.data="'.$LANG->getLL('online_checkdone').'.";
								</script>
							';
						}

						$this->content .= '
							</table>
						';

						$this->flushContent();
					} else if (!empty($this->online_update)) {
						$this->content .= '
							<br />
							<p id="progress-message">'.$LANG->getLL('online_wait').' ...</p>
							<br />
							<div style="width:100%; height:20px; border: 1px solid black;">
								<div id="progress-bar" style="float: left; width: 0%; height: 20px; background-color:green;">&nbsp;</div>
								<div id="transparent-bar" style="float: left; width: 100%; height: 20px; background-color:#FEFEFE;">&nbsp;</div>
							</div>
							<script language="javascript" type="text/javascript">
								document.getElementById("progress-bar").firstChild.data="";
								document.getElementById("transparent-bar").firstChild.data="";
							</script>
							<br />
							<br />
							<table border="0" cellpadding="2" cellspacing="2">
								<tr class="bgColor2">
									<td>'.$LANG->getLL('syntax_title').'</td>
									<td>'.$LANG->getLL('syntax_status').'</td>
								</tr>
						';

						$this->flushContent();

						if ($this->onlineUpdate()) {
							$this->content .= '
								<script language="javascript" type="text/javascript">
									document.getElementById("progress-message").firstChild.data="'.$LANG->getLL('online_updatedone').'.";
								</script>
							';
						}

						$this->content .= '
							</table>
						';

						$this->flushContent();
					}
				}
			break;
		}
	}
}

if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/mh_code_highlighter/mod1/index.php']) {
	include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/mh_code_highlighter/mod1/index.php']);
}

// Make instance:
$SOBE = t3lib_div::makeInstance('tx_mhcodehighlighter_module1');
$SOBE->init();

// Include files?
foreach($SOBE->include_once as $INC_FILE) include_once($INC_FILE);

$SOBE->main();
$SOBE->content = '';
$SOBE->printContent();

?>
