<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2008 Marc Hoersken <info@marc-hoersken.de>
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

if (version_compare(TYPO3_branch, '6.2', '<')) {
	require_once(PATH_tslib.'class.tslib_pibase.php');
}

require_once(t3lib_extMgm::siteRelPath('mh_code_highlighter').'class.tx_mhcodehighlighter_geshi.php');

/**
 * Plugin 'Sourcecode' for the 'mh_code_highlighter' extension.
 *
 * @author Marc Hoersken <info@marc-hoersken.de>
 * @package TYPO3
 * @subpackage tx_mhcodehighlighter
 */
class tx_mhcodehighlighter_pi1 extends tslib_pibase {
	var $prefixId = 'tx_mhcodehighlighter_pi1';
	var $scriptRelPath = 'pi1/class.tx_mhcodehighlighter_pi1.php'; // Path to this script relative to the extension dir.
	var $extKey = 'mh_code_highlighter'; // The extension key.
	var $pi_checkCHash = true;
	 
	/**
	 * The main method of the PlugIn
	 *
	 * @param string  $content: The PlugIn content
	 * @param array  $conf: The PlugIn configuration
	 * @return  string  The content that is displayed on the website
	 */
	function main($content, $conf) {
		$this->conf = $conf;
		$this->pi_setPiVarDefaults();
		$this->pi_loadLL();

		$GLOBALS['TSFE']->setJS($this->extKey, '
			function selectCode(a) {
				var e = (a.parentNode.parentNode.getElementsByTagName("OL"))[0];
				if (!e) {
					var e = (a.parentNode.parentNode.getElementsByTagName("PRE"))[0];
				}
				if (window.getSelection) {
					var s = window.getSelection();
					if (s.setBaseAndExtent) {
						s.setBaseAndExtent(e, 0, e, e.innerText.length - 1);
					} else {
						var r = document.createRange();
						r.selectNodeContents(e);
						s.removeAllRanges();
						s.addRange(r);
					}
				}
				else if (document.getSelection) {
					var s = document.getSelection();
					var r = document.createRange();
					r.selectNodeContents(e);
					s.removeAllRanges();
					s.addRange(r);
				}
				else if (document.selection) {
					var r = document.body.createTextRange();
					r.moveToElementText(e);
					r.select();
				}
			}
			function toggleLineNumbers(a) {
				var e = (a.parentNode.parentNode.getElementsByTagName("OL"))[0];
				var t = e.style.listStyleType;
				if (t == "none") {
					e.style.marginLeft="40px";
					e.style.listStyleType = "decimal";
				} else {
					e.style.marginLeft="0px";
					e.style.listStyleType = "none";
				}
			}
			');
		 
		$content = $this->renderContent();
		 
		return $this->pi_wrapInBaseClass($content);
	}
	 
	/**
	 * The content method of the PlugIn
	 *
	 * @return string The content that is displayed on the website
	 */
	function renderContent() {
		global $TYPO3_DB;

		$template = $this->cObj->fileResource($this->conf['templateFile']);
		$title = $this->cObj->data['header'];

		if ($this->cObj->data['list_type'] == 'mh_code_highlighter_pi1') {
			$geshiCode = $this->cObj->data['tx_mhcodehighlighter_geshi_code'];
			$geshiLang = $this->cObj->data['tx_mhcodehighlighter_geshi_lang'];
			$geshiSelectAll = $this->cObj->data['tx_mhcodehighlighter_geshi_selectall'];
			$geshiLineNumbering = $this->cObj->data['tx_mhcodehighlighter_geshi_linenumbering'];
			$geshiTabWidth = $this->cObj->data['tx_mhcodehighlighter_geshi_tabwidth'];
		} else {
			$geshiCode = $this->conf['value'];
			$geshiLang = $this->conf['config.']['lang'];
			$geshiSelectAll = $this->conf['config.']['selectall'];
			$geshiLineNumbering = $this->conf['config.']['linenumbering'];
			$geshiTabWidth = $this->conf['config.']['tabwidth'];
		}

		if (is_numeric($geshiLang)) {
			$geshiLang_integer = intval($geshiLang);
			$res = $TYPO3_DB->exec_SELECTquery('uid,geshi_lang,geshi_file', 'tx_mhcodehighlighter_languages', 'uid = '.$geshiLang_integer);
			$num = $TYPO3_DB->sql_num_rows($res);
			if ($num > 0) {
				$row = $TYPO3_DB->sql_fetch_assoc($res);
				if (is_array($row)) {
					if (($row['uid'] == $geshiLang) && !empty($row['geshi_lang'])) {
						$geshiLang = $row['geshi_lang'];
						$geshiFile = $row['geshi_file'];
					}
				}
			}
		} else {
			$geshiLang_quoted = $TYPO3_DB->fullQuoteStr($geshiLang, 'tx_mhcodehighlighter_languages');
			$res = $TYPO3_DB->exec_SELECTquery('uid,geshi_lang,geshi_file', 'tx_mhcodehighlighter_languages', 'geshi_file = '.$geshiLang_quoted);
			$num = $TYPO3_DB->sql_num_rows($res);
			if ($num > 0) {
				$row = $TYPO3_DB->sql_fetch_assoc($res);
				if (is_array($row)) {
					if (($row['geshi_file'] == $geshiLang) && !empty($row['geshi_lang'])) {
						$geshiFile = $row['geshi_file'];
					}
				}
			}
		}
		 
		if (!empty($geshiCode) && $geshiCode[0] == "\n") {
			$geshiCode = substr($geshiCode, 1);
		}

		if (!empty($geshiLang) && file_exists(t3lib_extMgm::siteRelPath('mh_code_highlighter').'res/'.$geshiFile)) {
			$geshiInstance = t3lib_div::makeInstance('tx_mhcodehighlighter_geshi');
			$geshi = new $geshiInstance();
			$geshi->set_source($geshiCode);
			$geshi->set_language_path(t3lib_extMgm::siteRelPath('mh_code_highlighter').'res/');
			$geshi->set_language(str_replace('.php', '', $geshiFile));
			$geshi->set_line_style($this->conf['_CSS_LINE_1_STYLE'], $this->conf['_CSS_LINE_2_STYLE']);
			if ($geshiLineNumbering) {
				$geshi->enable_line_numbers(GESHI_FANCY_LINE_NUMBERS);
				$geshi->set_header_type(GESHI_HEADER_NONE);
			} else {
				$geshi->enable_line_numbers(GESHI_NO_LINE_NUMBERS);
				$geshi->set_header_type(GESHI_HEADER_PRE);
			}
			if ($geshiTabWidth > 0) {
				$geshi->set_tab_width($geshiTabWidth);
			} else {
				$geshi->set_tab_width(4);
			}
			$content = @$geshi->parse_code();
			$content = str_replace(array('<br />', '<br>'), '', $content);
		} else {
			$content = htmlentities($geshiCode);
		}

		$markerArray = array();
		$markerArray['###LANG_SELECTALL###'] = $this->pi_getLL('selectall');
		$subpartSelectAll = $this->cObj->getSubpart($template, '###LINK_SELECTALL###');
		$contentSelectAll = $this->cObj->substituteMarkerArray($subpartSelectAll, $markerArray);
		 
		$markerArray = array();
		$markerArray['###LANG_LINENUMBERS###'] = $this->pi_getLL('linenumbers');
		$subpartLineNumbering = $this->cObj->getSubpart($template, '###LINK_LINENUMBERING###');
		$contentLineNumbering = $this->cObj->substituteMarkerArray($subpartLineNumbering, $markerArray);
		 
		$markerArray = array();
		$markerArray['###CONTENT###'] = $content;
		$markerArray['###LINKS###'] = '';
		$markerArray['###LANG_CODE###'] = $this->pi_getLL('code');
		 
		if ($geshiSelectAll || $geshiLineNumbering) {
			$markerArray['###LINKS###'] .= '&nbsp;';
		}
		if ($geshiSelectAll) {
			$markerArray['###LINKS###'] .= $contentSelectAll;
		}
		if ($geshiSelectAll && $geshiLineNumbering) {
			$markerArray['###LINKS###'] .= '&nbsp;|&nbsp;';
		}
		if ($geshiLineNumbering) {
			$markerArray['###LINKS###'] .= $contentLineNumbering;
		}
		 
		$subpart = $this->cObj->getSubpart($template, '###TEMPLATE###');
		$content = $this->cObj->substituteMarkerArray($subpart, $markerArray);
		 
		return $content;
	}
}

if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/mh_code_highlighter/pi1/class.tx_mhcodehighlighter_pi1.php']) {
	include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/mh_code_highlighter/pi1/class.tx_mhcodehighlighter_pi1.php']);
}

?>
